/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef BITEXCEPTION_H
#define BITEXCEPTION_H

/**
 * @defgroup exceptions "Exceptions"
 */

#include <stdexcept>

namespace bit
  {

  /**
   * Base class for all bit library exceptions.
   * @author Rick L Vinyard Jr
   * @ingroup exceptions
   */
  class bit_exception: public std::runtime_error
    {
    public:
      bit_exception(const std::string s="bit: Unknown exception."):
          std::runtime_error(s),
          m_exception_string(s)
      {}

      ~bit_exception() throw ()
      { }

      virtual const char * what () const throw ()
      {
        return m_exception_string.c_str();
      }

    protected:
      std::string m_exception_string;
    }
  ;

  namespace exception
    {

    /**
    * @author Rick L Vinyard Jr
    * @ingroup exceptions
     */
    class invalid_iterator: public bit_exception
    {
    public:
    invalid_iterator(): bit_exception("bit: Attempted to use invalid iterator")
      { }
    }
    ;

    /**
     * @author Rick L Vinyard Jr
     * @ingroup exceptions
     */
    class invalid_container_op: public bit_exception
    {
      public:
        invalid_container_op(): bit_exception("bit: Using container methods on a FieldBase descendant that is not valid for that descendant.")
        { }
    }
    ;

    /**
    * @author Rick L Vinyard Jr
    * @ingroup exceptions
     */
    class name: public bit_exception
      {
      public:
        name(): bit_exception("bit: Name identifiers may not contain the following characters: .[]")
        { }
        name(const std::string s): bit_exception(s)
        {
          m_exception_string = "bit: The name \"" + s + "\" is invalid. Name identifiers may not contain the following characters: .[]";
        }
      }
    ;

    /**
     * @author Rick L Vinyard Jr
     * @ingroup exceptions
     */
    class no_record: public bit_exception
      {
      public:
        no_record(): bit_exception("bit:FieldBuffer: record() method called, but this instance does not have a record.")
        { }
      }
    ;

    /**
     * @author Rick L Vinyard Jr
     * @ingroup exceptions
     */
    class invalid_index: public bit_exception
    {
      public:
        invalid_index(): bit_exception("bit: Invalid index.")
        { }
    }
    ;

    }

}

#include <bit/except_field.h>
#include <bit/except_record.h>
#include <bit/except_indices.h>
#include <bit/except_type.h>

#endif

